rm(list=ls())
gc(TRUE)

##====================== Initialisation ========================####
if (TRUE){
  setwd("C:/GoogleDrive/MyDocs/Annastiina/Topics/MTVGJR_MGARCH/Project2_Building")        # Anna's work PC  - GOOGLE DRIVE
  #setwd("~/Project2_Building")                                         
  
  source("clsTV.r")
  source("clsGARCH.r")
  source("clsMTVGARCH.r")
  library(parallel)
  library(RevoUtilsMath)
  maxThreads <- detectCores()-1
  setMKLthreads(maxThreads)  
}
##====================== Initialisation ========================##


##====================== Data Load =============================####
if (TRUE){
  # Read AUS_4Banks data from saved file
  mydata <- readRDS("AUS_bankreturns_1992_2020.RDS")
  dates <- mydata$date
  
  # Create scaled-up & de-meaned individual data series vectors (Percentage Returns):
  e_nab <- mydata$return_nab * 100 
  e_nab <- e_nab - mean(e_nab)
  e <- e_nab
  
  # Plot the returns
  ptitle <- "NAB Returns"
  plot(dates,e,type="l",main=ptitle)
  
  # Read in the Garch'd Reference Data
  RefData <- readRDS("Gen_Refdata/RefData_withGarch_NAB_2020.RDS")
  
  # Tidy Up
  rm(mydata)
  gc(TRUE)
}
##====================== Data Load =============================##


##   
##============  General Specification Method    ===============####
##
##   1. Plot the local Variance and visually observe any transitions
##   2. Calculate a set of *indicative* garch parameters using different window lengths & kurtosis
##   3. Identify the first "single" transition start & end:
##     4. Generate refData0 using TV$delta0 = var(e[start:end])
##     5. Test for a first transition in this subset of the time series 
##     6. Generate refData1 using TV$delta0 & TV$pars = TV estimates found above
##     7. Test for a second transition in this subset of the time series, and if none is found, move on to the next subset 
##   8. Repeat steps 4 - 7 for each subsequent data subset where we think there may be a transition
##   9. Complete the specification by adding the individual sub-sections together, re-estimating & re-testing
##  10. Consider if any of the transitions could be re-parameterised as a double-transition to reduce the total number of parameters
##  11. Re-estimate the final model and calculate & check the standard errors

##==============================================================##


# Plot returns using index as x-axis:
ptitle <- "NAB Returns"
plot(e,type="l",main = ptitle)
abline(h=seq(-0.1,0.1,0.05), v=seq(0,7000,500),col="lightgrey")
lines(e)



##==============================================================##
##  TV.ORDER 0, obs 1:7108  ####
##==============================================================##

if (TRUE) {

  e <- e_nab
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  TV <- tv(st,tvshape$delta0only)
  TV <- estimateTV(e,TV)
  
  # Now Test against NAB
  TV <- setTaylorOrder(1,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "NAB.ORD0.1_1-7108.RDS"
  simcontrol$numLoops <- 1000
  simcontrol$numCores <- 10
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)

  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}  
##==============================================================##
##  TV - AUS_4Banks/NAB ---- ORDER 0.1 ---- obs 1:7108          ##

## Result: Pvals=[TR2:57%,Robust=31%] 
## Fail to Reject the null: H0=Model is sufficient
##  This is likely caused by issues with the optimiser 
##  where there are many transitions
##  So, try a smaller range... 
##==============================================================##



##==============================================================##
##  TV.ORDER 0, obs 1:4000  ####
##==============================================================##

if (TRUE) {
  
  e <- e_nab[1:4000]
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  TV <- tv(st,tvshape$delta0only)
  TV <- estimateTV(e,TV)
  
  # Now Test against NAB
  TV <- setTaylorOrder(3,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "NAB.ORD0.3_1-4000.RDS"
  simcontrol$numLoops <- 1200
  simcontrol$numCores <- 10
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)
  
  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}  # End: if (DoThis)...
##==============================================================##
##  TV - AUS_4Banks/NAB ---- obs 1:4000          
## ORDER 0.1 
## Result: Pvals=[TR2=88%,Robust=81%] 
## ORDER 0.2
## Result: Pvals=[TR2= 13%,Robust= 10%] 
## ORDER 0.3
## Result: Pvals=[TR2= 22%,Robust= 1.3%] 

## Reject the null: 
## Conclusion: Evidence of a transition here, but since the
##            order 1 & 2 didn't detect it, we will try increasing
##            search area first
##==============================================================##



##==============================================================##
##  TV.ORDER 0, obs 1:5000  ####
##==============================================================##

if (TRUE) {
  
  e <- e_nab[1:5000]
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  TV <- tv(st,tvshape$delta0only)
  TV <- estimateTV(e,TV)
  
  # Now Test against NAB
  TV <- setTaylorOrder(1,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "NAB.ORD0.1_1-5000.RDS"
  simcontrol$numLoops <- 1200
  simcontrol$numCores <- 10
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)
  
  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}  # End: if (DoThis)...
##==============================================================##
##  TV - AUS_4Banks/NAB ---- ---- obs 1:5000          

## ORDER 0.1 
## Result: Pvals=[TR2= 0.25%,Robust= 0.08%] 
## ORDER 0.2
## Result: Pvals=[TR2= %,Robust= %] 
## ORDER 0.3
## Result: Pvals=[TR2= %,Robust= %] 

## Reject the null:
## Conclusion: Strong evidence of a linear transition here

##==============================================================##



##==============================================================##
##  TV.ORDER 1, obs 1:5000  ####
##==============================================================##

if (TRUE) {
  
  e <- e_nab[1:5000]
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  # Add some estimation controls
  estCtrl <- list()
  estCtrl$calcSE <- TRUE
  estCtrl$verbose <- TRUE
  
  TV <- tv(st,tvshape$single)
  
  OptCtrl <- TV$optimcontrol
  OptCtrl$reltol <- 1e-9
  OptCtrl$ndeps <- rep(1e-7,TV@nr.pars)
  OptCtrl$parscale <- c(6,6,2,1)
  TV$optimcontrol <- OptCtrl
  
  TV$delta0 <- 0.1
  TV$pars[1,1] <- 0.05
  TV$pars[2,1] <- 2.05
  TV$pars[3,1] <- 0.25

  TV <- estimateTV(e,TV,estCtrl)
  
  summary(TV)
  plot(TV)
  
  # Now Test against NAB
  TV <- setTaylorOrder(2,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "NAB.ORD1.2_1-5000.RDS"
  simcontrol$numLoops <- 1200
  simcontrol$numCores <- 9
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)
  
  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
} 

##==============================================================##
##  TV - AUS_4Banks/NAB ---- ---- obs 1:5000    
# Estimation Results:
#   
#   Delta0 = 0 se0 =  NaN 
# 
#           st1      se1
# deltaN 0.000399 0.000018
# speedN 1.264998 0.046595
# locN1  0.386388 0.017454
# locN2        NA      NaN
# 
# Log-liklihood value:  14120.16
##==============================================================##
## ORDER 1.1 
## Result: Pvals=[TR2= NaN%,Robust= 25%] 
## ORDER 1.2
## Result: Pvals=[TR2= NaN%,Robust= 7.23%] 
## ORDER 1.3
## Result: Pvals=[TR2=%,Robust=%] 

## Fail to Reject the null
## Conclusion: Weak evidence of a transition here, but since the
##            order 1 didn't detect it, we will try increasing
##            search area again
##==============================================================##



##==============================================================##
##  TV.ORDER 0, obs 4001:7108  ####
##==============================================================##

if (TRUE) {
  
  e <- e_nab[4001:7108]
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  TV <- tv(st,tvshape$delta0only)
  TV <- estimateTV(e,TV,estCtrl)

  # Now Test against NAB
  TV <- setTaylorOrder(1,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "NAB.ORD0.1_4001-7108.RDS"
  simcontrol$numLoops <- 1200
  simcontrol$numCores <- 3
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)
  
  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}  # End: if (DoThis)...
##==============================================================##
##  TV - AUS_4Banks/NAB ---- ---- obs 4001:7108
## ORDER 0.1 
## Result: Pvals=[TR2= 0%,Robust= 0%] 
## ORDER 0.2
## Result: Pvals=[TR2= 84%,Robust= 90%] 
## ORDER 0.3
## Result: Pvals=[TR2= 84%,Robust= 73%] 

## Reject the null
## Conclusion: Strong evidence of a transition here

##==============================================================##



##==============================================================##
##  TV.ORDER 1, obs 4001:7108  ####
##==============================================================##

if (TRUE) {
  
  e <- e_nab[4001:7108]
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  # Add some estimation controls
  estCtrl <- list()
  estCtrl$calcSE <- TRUE
  estCtrl$verbose <- TRUE
  
  TV <- tv(st,tvshape$single)
  
  OptCtrl <- TV$optimcontrol
  OptCtrl$reltol <- 1e-9
  OptCtrl$ndeps <- c(1e-7,1e-7,1e-7,1e-7)
  #OptCtrl$parscale <- c(6,6,2,1)
  TV$optimcontrol <- OptCtrl
  
  TV$delta0 <- 0.5
  TV$pars[1,1] <- 0.05
  TV$pars[2,1] <- 3.05
  TV$pars[3,1] <- 0.5
  
  TV <- estimateTV(e,TV,estCtrl)
  
  summary(TV)
  plot(TV)
  
  # Now Test against NAB
  TV <- setTaylorOrder(2,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "NAB.ORD1.2_4001-7108.RDS"
  simcontrol$numLoops <- 1200
  simcontrol$numCores <- 9
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)
  
  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}

##==============================================================##

# Estimation Results:
#   
# Delta0 = 0.001545 se0 =  0.000328 
# 
#             st1      se1
# deltaN -0.001425 0.000328
# speedN  2.691393 0.124677
# locN1   0.083984 0.030331
# locN2         NA      NaN
# 
# Log-liklihood value:  8915.693
##==============================================================##

##==============================================================##
##  TV - AUS_4Banks/NAB ---- ---- obs 4001:7108          
## ORDER 1.1 
## Result: Pvals=[TR2=39 %,Robust= 33%] 
## ORDER 1.2
## Result: Pvals=[TR2= 59%,Robust= 64%] 
## ORDER 1.3
## Result: Pvals=[TR2= %,Robust= %] 

## Fail to Reject the null
## Conclusion: No Evidence of a transition here

##==============================================================##


##==============================================================##
##  TV.ORDER 2, obs 1:7108  ####
##==============================================================##

if (TRUE) {
  
  e <- e_nab[1:7108]
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  # Add some estimation controls
  estCtrl <- list()
  estCtrl$calcSE <- TRUE
  estCtrl$verbose <- TRUE
  
  TV <- tv(st,c(tvshape$single,tvshape$double1loc))
  
  OptCtrl <- TV$optimcontrol
  OptCtrl$reltol <- 1e-9
  OptCtrl$ndeps <- c(1e-7,1e-7,1e-7,1e-7,1e-7,1e-7,1e-7)
  #OptCtrl$parscale <- c(6,3,1,1,3,1,1)
  OptCtrl$parscale <- c(3,3,1,1,3,1,1)
  TV$optimcontrol <- OptCtrl
  
  TV$delta0 <- 1
  TV$pars["deltaN",] <- c(1,1)
  TV$pars["speedN",1] <- 3.05
  TV$pars["locN1",1] <- 0.2
  
  TV$pars["speedN",2] <- 3.05
  TV$pars["locN1",2] <- 0.6

  TV <- estimateTV(e,TV,estCtrl)
  
  summary(TV)
  plot(TV)
  
  # Fix the std errors
  h0 <- TV$Estimated$hessian
  h1 <- h0[-6,-6]
  se1 <- try(stdErrors <- sqrt(-diag(qr.solve(h1))))
  h2 <- h1[-3,-3]
  se2 <- try(stdErrors <- sqrt(-diag(qr.solve(h2))))
  
  
  # Now Test against NAB
  TV <- setTaylorOrder(2,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "NAB.ORD2.2_1-7108.RDS"
  simcontrol$numLoops <- 1200
  simcontrol$numCores <- 4
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)
  
  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}

##==============================================================##

# Estimation Results:
#   
#   Delta0 = 15.2539 se0 =  0.061529 
# 
#               st1      se1 sig        st2      se2   sig.1
#   deltaN 0.433884 0.051305 *   -14.189119 0.067410   ***
#   speedN 6.999964 0.182940 **    7.000000 0.000000   ***
#   locN1  0.201900 0.001770 **    0.601815 0.001578   ***
#   locN2        NA      NaN             NA      NaN      
# 
# Log-liklihood value:  -11794.82
##==============================================================##


##==============================================================##
##  TV - AUS_4Banks/NAB ---- ---- obs 1:7108          
## ORDER 2.1 
## Result: Pvals=[TR2= 55%,Robust= 26%] 
## ORDER 2.2
## Result: Pvals=[TR2= 62%,Robust= 51%] 
## ORDER 2.3
## Result: Pvals=[TR2= %,Robust= %] 

## Reject the null
## Conclusion: No Evidence of another transition here
##             Model is fully specified
##==============================================================##



##==============================================================##
##  TV.ORDER 2, MTVGJR Model - Manual Estimation   ####
##==============================================================##

if (TRUE) {
  
  ##==============================================================##
  ##  TV - AUS_4Banks\NAB ---- ORDER 3 ---- obs 1:7027            ##
  ##

  
  # Create e, TV1 & G1 
  # make an original copy of the data:
  e_orig <- e
  estCtrl <- list(calcSE = TRUE, verbose = TRUE)
  st <- 1:length(e)/length(e)
  
  TV1 <- tv(st,c(tvshape$single,tvshape$single,tvshape$single))
  #startpars <- c(1,3,0.4,5,3,0.5,-5,3,0.7)
  #parScale <- c(1,1,1,0.3,3,1,0.3,3,1,0.3)
  
  TV1$delta0 <- 1
  # TV1$pars["deltaN",] <- c(-1.0,12.0,-10.0)   
  # TV1$pars["deltaN",] <- c(-1.0,7.0,-7.0)  # -11721.46
  # TV1$pars["speedN",] <- c(3.0,3.0,3.0)
  # TV1$pars["locN1",] <- c(0.4,0.6,0.8)
  TV1$pars["deltaN",] <- c(-1.0,8.0,-8.0)  # -11720.75
  TV1$pars["speedN",] <- c(3.0,3.0,3.0)
  TV1$pars["locN1",] <- c(0.4,0.6,0.7)
  
    
  optCtrl <- TV1$optimcontrol
  optCtrl$reltol <- 1e-9
  optCtrl$ndeps <- c(1e-5, 1e-7,1e-7,1e-5, 1e-5,1e-5,1e-5, 1e-5,1e-5,1e-5)
  optCtrl$parscale <- c(1, 2,1,1, 3,2,1, 3,1,1)
  TV1$optimcontrol <- optCtrl
  
  TV1 <- estimateTV(e,TV1,estCtrl)
  summary(TV1)  
  plot(TV1)
  
  G1 <- garch(garchtype$gjr)
  
  OptCtrl <- G1$optimcontrol
  OptCtrl$reltol <- 1e-7
  OptCtrl$ndeps <- c(1e-5,1e-5,1e-5,1e-5)
  OptCtrl$parscale <- c(1,1,9,1)
  G1$optimcontrol <- OptCtrl
  
  G1$pars["omega",1] <- 0.01
  G1$pars["alpha",1] <- 0.06
  G1$pars["beta",1] <- 0.8
  G1$pars["gamma",1] <- 0.03
  
  G1 <- estimateGARCH(e,G1,estCtrl)
  G1 <- estimateGARCH(e/sqrt(TV1@g),G1,estCtrl)
  summary(G1)
  plot(G1)
  
  MTV1 <- mtvgarch(e,TV1,G1)
  
  ###  START THE PAIR_WISE ESTIMATION HERE  ###
  
  # Filter out Garch & estimate TV component
  if(TRUE) {
    
    lG1 <- MTV1@garch                # For first iteration
  #  lG1 <- MTV1$results[[1]]$garch   # For subsequent iterations - update the index!
    e <- e_orig/sqrt(lG1@h)
    
    lT1 <- MTV1@tv                # For first iteration
    #lT1 <- MTV1$results[[1]]$tv   # For subsequent iterations - update the index!
    
    summary(lT1)
    ll.target <- getTargetValue(e,lT1)
    
    lT1$delta0 <- lT1$Estimated$delta0
    lT1$pars <- lT1$Estimated$pars 
    
    lT1$pars["deltaN",] <-   c(-1.0,30.0,-25.0)#c(-1.0,20.0,-15.0) #c(-1.0,9.0,-8.0)
    lT1$pars["speedN",] <- c(3.0,3.0,3.0)
    lT1$pars["locN1",] <- c(0.4,0.6,0.7)
    #lT1$pars["locN2",] <- 0
    
    optCtrl <- lT1$optimcontrol
    optCtrl$reltol <- 1e-9
    optCtrl$ndeps <- c(1e-7,1e-5,1e-7,1e-7,1e-5,1e-7,1e-7,1e-5,1e-7)
    optCtrl$parscale <-     c(3,1,1,60,1,1,60,1,1) #c(3,1,1,5,1,1,5,1,1) #c(3,1,1,3,1,1,3,1,1)
    lT1$optimcontrol <- optCtrl
    
    lT1 <- estimateTV(e,lT1,estCtrl)
    summary(lT1)  
    plot(lT1)
    
    # Check that the LL is higher than the target:
    lT1$Estimated$value > ll.target 
    
  }
  
  # Filter out TV & estimate GARCH component
  if(TRUE) {
    e <- e_orig/sqrt(lT1@g)
    
    lG1 <- MTV1@garch                # For first iteration
    #lG1 <- MTV1$results[[1]]$garch   # For subsequent iterations - update the index!
    summary(lG1)
    ll.target <- getTargetValue(e,garchObj = lG1)
    lG1$pars <- lG1$Estimated$pars
    
    lG1$pars["omega",1] <- 0.10
    lG1$pars["alpha",1] <- 0.05
    lG1$pars["beta",1] <- 0.80
    lG1$pars["gamma",1] <- 0.02
    
    optCtrl <- lG1$optimcontrol
    optCtrl$reltol <- 1e-9
    optCtrl$ndeps <- c(1e-7,1e-7,1e-7,1e-7)
    optCtrl$parscale <- c(1,1,9,1)
    lG1$optimcontrol <- optCtrl
    
    lG1 <- estimateGARCH(e,lG1,estCtrl)
    summary(lG1)  
    plot(lG1)
    
    # Check that the LL is higher than the target:
    lG1$Estimated$value > ll.target 
    
  }
  
  # Now call the calculate liklihood method for mtvgarch
  
  MTV1 <- calcLoglik(MTV1,lT1,lG1)
  
  # Pick the best specification & write it to the Estimated List:
  MTV1$Estimated[[1]] <- MTV1$results[[1]]
  
  
  # Save the estimated model
  saveRDS(MTV1,"Output/NAB_mtvgjr_manual.RDS")
}

##==============================================================##
##                            THE END
##==============================================================##
