rm(list=ls())
gc(TRUE)

##====================== Initialisation ========================####
if (TRUE){
  setwd("C:/GoogleDrive/MyDocs/Annastiina/Topics/MTVGJR_MGARCH/Project2_Building")        # Anna's work PC  - GOOGLE DRIVE
  #setwd("~/Project2_Building")                                         
  
  ## Note:  
  ##  This file was re-written after the univariate TV-Specification work was completed
  ##  The earlier clsTV, clsGARCH, etc. files have now been compiled into the MTVGARCH package
  ##  Methods are now provided for estimateTV(), estimateGARCH() and estimateTVGARCH()
  
  library(MTVGARCH)

}
##====================== Initialisation ========================##


##====================== Data Load =============================####
if (TRUE){
  # Read AUS_4Banks data from saved file
  mydata <- readRDS("AUS_bankreturns_1992_2020.RDS")
  dates <- mydata$date

  e_anz <- 100 * ( mydata$return_anz - mean(mydata$return_anz) )
  e_cba <- 100 * ( mydata$return_cba - mean(mydata$return_cba) )
  e_nab <- 100 * ( mydata$return_nab - mean(mydata$return_nab) )
  e_wbc <- 100 * ( mydata$return_wbc - mean(mydata$return_wbc) )

}
##====================== Data Load =============================##


## ANZ Setup
if(F){
  e <- e_anz
  fileName <- "Output/tvgjr_ANZ.RDS"
  # Plot the returns
  ptitle <- "ANZ Returns"
  plot(dates,e,type="l",main=ptitle)

  St <- (1:length(e))/length(e)
  Shape <- c(tvshape$single,tvshape$single,tvshape$single)

  tv1 <- tv(St,Shape)

  tv1$delta0 <- 1
  tv1$pars["deltaN",] <- c(0.75,5.0,-5.0)
  tv1$pars["speedN",] <- c(3.0,3.0,3.0)
  tv1$pars["locN1",] <- c(0.4,0.5,0.7)

  optCtrl <- tv1$optimcontrol
  optCtrl$reltol <- 1e-9
  optCtrl$ndeps <- c(1e-5, 1e-7,1e-7,1e-5, 1e-6,1e-5,1e-5, 1e-6,1e-5,1e-5)
  optCtrl$parscale <- c(1, 2,1,1, 3,2,1, 3,1,1)
  tv1$optimcontrol <- optCtrl
}

## CBA Setup
if(F) {
  e <- e_cba
  fileName <- "Output/tvgjr_CBA.RDS"
  # Plot the returns
  ptitle <- "CBA Returns"
  plot(dates,e,type="l",main=ptitle)

  St <- (1:length(e))/length(e)
  Shape <- c(tvshape$single,tvshape$single,tvshape$single)

  tv1 <- tv(St,Shape)

  optCtrl <- tv1$optimcontrol
  optCtrl$reltol <- 1e-9
  optCtrl$parscale <- c(3, 3,1,1, 3,3,1, 3,3,1)
  optCtrl$ndeps <- rep(1e-5, tv1@nr.pars)
  #optCtrl$ndeps <- c(1e-5, 1e-5,1e-7,1e-5, 1e-7,1e-5,1e-5, 1e-7,1e-5,1e-5)
  tv1$optimcontrol <- optCtrl

  tv1$delta0 <- 1.0
  tv1$pars["deltaN",] <- c(1, 3, -3)
  tv1$pars["speedN",] <- c(1, 3, 3)
  tv1$pars["locN1",] <- c(0.3, 0.6, 0.7)

}

## NAB Setup
if(F) {
  e <- e_nab
  fileName <- "Output/tvgjr_NAB.RDS"
  # Plot the returns
  ptitle <- "NAB Returns"
  plot(dates,e,type="l",main=ptitle)

  St <- (1:length(e))/length(e)
  Shape <- c(tvshape$double1loc,tvshape$single,tvshape$single)

  tv1 <- tv(St,Shape)

  optCtrl <- tv1$optimcontrol
  optCtrl$reltol <- 1e-9
  optCtrl$parscale <- c(3, 3,3,0.05, 5,3,0.05, 5,3,0.05)
  #optCtrl$ndeps <- rep(1e-5, tv1@nr.pars)
  optCtrl$ndeps <- c(1e-5, 1e-5,1e-7,1e-5, 1e-7,1e-5,1e-5, 1e-7,1e-5,1e-5)
  tv1$optimcontrol <- optCtrl

  tv1$delta0 <- 1.0
  tv1$pars["deltaN",] <- c(1, 1, 1)
  tv1$pars["speedN",] <- c(3, 3, 3)
  tv1$pars["locN1",] <- c(0.15, 0.6, 0.8)

}

## WBC Setup
if(F) {
  e <- e_wbc
  fileName <- "Output/tvgjr_WBC.RDS"
  # Plot the returns
  ptitle <- "WBC Returns"
  plot(dates,e,type="l",main=ptitle)

  St <- (1:length(e))/length(e)
  Shape <- c(tvshape$single,tvshape$single,tvshape$single)

  tv1 <- tv(St,Shape)

  optCtrl <- tv1$optimcontrol
  optCtrl$reltol <- 1e-7
  optCtrl$parscale <- c(1, 1,1,0.2, 3,1,0.2, 3,1,0.2)
  optCtrl$ndeps <- rep(1e-5, tv1@nr.pars)
  tv1$optimcontrol <- optCtrl

  tv1$delta0 <- 1.0
  tv1$pars["deltaN",] <- c(1, 3, -3)
  tv1$pars["speedN",] <- c(3, 3, 3)
  tv1$pars["locN1",] <- c(0.4, 0.6, 0.8)

}

## --- Start the multiplicative TV-GARCH Estimation -- ####


# Run one of the 4 Setup sections above to populate the 'e', 'tv1' & 'fileName' variables.

estCtrl <- list(calcSE = TRUE, verbose = TRUE)
tv1 <- estimateTV(e,tv1,estCtrl)
plot(tv1)
summary(tv1)

g1 <- garch(garchtype$gjr)
g1 <- estimateGARCH(e,g1,estCtrl)
plot(g1)
summary(g1)

tvg1 <- tvgarch(tv1,garchtype$gjr)
tvg1 <- estimateTVGARCH(e,tvg1,estCtrl)

summary(tvg1)

## Note: If you get warnings like 'In sqrt(-diag(qr.solve(tmp$hessian))) : NaNs produced'
##       This indicates there were numerical issues inverting the hessian
##       Often these can be resolved my changing the starting parameters and adjusting the
##       optimizer controls.
##       We recommend re-running the estimations until standard errors are calculated for all parameters,
##       ensuring that the log-likelihood does not move to far from its maximum value.

# Compare TV-only & Garch-Only estimates with the Multiplicative model:
if(T){
  plot(tv1)
  lines(sqrt(tvg1$Estimated$tv$g),lty=2, col="red")

  summary(tvg1)
  summary(tv1)
  
  plot(g1)
  lines(sqrt(tvg1$Estimated$garch$h), lty=2, col="red")

  summary(tvg1)
  summary(g1)

}

# Save tvgarch object - !! Change variable name first !!
tvg_WBC <- tvg1
saveRDS(tvg_WBC,fileName)


##==============================================================##
##                            THE END
##==============================================================##

