rm(list=ls())
gc(TRUE)

##====================== Initialisation ========================####
if (TRUE){
  setwd("C:/GoogleDrive/MyDocs/Annastiina/Topics/MTVGJR_MGARCH/Project2_Building")        # Anna's work PC  - GOOGLE DRIVE
  #setwd("~/Project2_Building")  
  
  ## Note:  
  ##  This file was re-written after the univariate TV-Specification work was completed
  ##  The earlier clsTV, clsGARCH, etc. files have now been compiled into the MTVGARCH package
  
  library(MTVGARCH)

}
##====================== Initialisation ========================##


##====================== Data Load =============================####
if (TRUE){
  # Read AUS_4Banks data from saved file
  mydata <- readRDS("AUS_bankreturns_1992_2020.RDS")
  dates <- mydata$date

  e_anz <- 100 * ( mydata$return_anz - mean(mydata$return_anz) )
  e_cba <- 100 * ( mydata$return_cba - mean(mydata$return_cba) )
  e_nab <- 100 * ( mydata$return_nab - mean(mydata$return_nab) )
  e_wbc <- 100 * ( mydata$return_wbc - mean(mydata$return_wbc) )

  ## GJR_GARCH MODELS:
  tvgANZ <- readRDS("Output/tvgjr_ANZ.RDS")
  tvgCBA <- readRDS("Output/tvgjr_CBA.RDS")
  tvgNAB <- readRDS("Output/tvgjr_NAB.RDS")
  tvgWBC <- readRDS("Output/tvgjr_WBC.RDS")

  ## GENERAL_GARCH MODELS:
  tvgANZ <- readRDS("Output/tvgarch_ANZ.RDS")
  tvgCBA <- readRDS("Output/tvgarch_CBA.RDS")
  tvgNAB <- readRDS("Output/tvgarch_NAB.RDS")
  tvgWBC <- readRDS("Output/tvgarch_WBC.RDS")

  # Tidy Up
  rm(mydata)
  gc(TRUE)
}
##====================== Data Load =============================##


##========== Multivariate MTV-GJR model ==========####

# Step 1. Create a multivariate MTV-GJR object:

mydata <- list(tvgANZ,tvgCBA,tvgNAB,tvgWBC)
myNames <- c("ANZ","CBA","NAB","WBC")
mtvg_AusBanks <- mtvgarch(mydata,myNames)


# Step 2. Put the data in a matrix - and filter it using the mtvgjr object

e <- cbind(e_anz,e_cba,e_nab,e_wbc)
z <- filterData(e,mtvg_AusBanks)

##========== CCC model ==========####

# Step 3. Create a CCC object to be used as the Alternative Hypothesis in tests for STCC

CCC <- ccc(mtvg_AusBanks)
CCC <- estimateCCC(z,CCC,estControl)

ALT <- list()
ALT$st <- (1:CCC@Tobs)/CCC@Tobs


##========== LM Tests for Constant Correlation ==========####

# Repeat tests for all test-orders 1..4
testorder <- 1
LMp <- test.CCCParsim(e,CCC,ALT,testorder)
LMs <- test.CCCvSTCC1(e,CCC,ALT,testorder)

###

H0 <- CCC
H1 <- ALT
testOrder <- 1

##========== STCC model (1 transition) ==========####

# Step 4. Create an STCC1 object for our model

STCC1 <- stcc1(mtvg_AusBanks)

# Note:  Overwrite st, shape, speed, loc before estimating if needed:
# You will need to use the set.St() method, since st is hidden in a slot

STCC1$shape <- corrshape$single
STCC1$pars["speed"] <- 2
STCC1$pars["loc1"] <- 0.5

estControl <- list(calcSE = TRUE,verbose = TRUE)

timestamp()
STCC1 <- estimateSTCC1(z,STCC1,estControl)
timestamp()


fileName <- "Output/AUS_4Banks_STCC1.RDS"

saveRDS(STCC1,fileName)
View(STCC1)

#STCC1 <- readRDS("Output_OCT2020/AUS_4Banks_STCC_20201028.RDS")

##====================== Quick Plot of STCC =========================####


 plot(STCC1$Estimated$Pt[,1],type='l',col="black",ylim=c(0.45,0.85),panel.first = grid(),ylab="STCC")
lines(STCC1$Estimated$Pt[,2],col="red")
lines(STCC1$Estimated$Pt[,3],col="green")
lines(STCC1$Estimated$Pt[,4],col="blue")
lines(STCC1$Estimated$Pt[,5],col="darkgrey",lwd=2)
lines(STCC1$Estimated$Pt[,6],col="yellow",lwd=2)

# Add a Legend
legendText <- c("Black     ANZ:CBA ","Red       ANZ:NAB ","Green    ANZ:WBC ")
legendText <- c(legendText,"Blue      CBA:NAB ","Grey      CBA:WBC ","Yellow   NAB:WBC ")
legend("topleft",legend = legendText)


##============================THE END  ========================####


