
# Clear the Environment
try(rm(list=ls()))


## Source the classes
source("clsTV.r")
source("clsGARCH.r")
source("clsMTVGARCH.r")

#Create dummy data
set.seed(1999)
e <- rnorm(1000,0,2)
plot(e,type='l')

#### Create an MTVGARCH object, using a TV & GARCH ####
if(TRUE){
  
  st <- (1:1000)/1000
  myShape <- tvshape$single
  TV1 <- tv(st,myShape)
  estCtrl <- list(calcSE = TRUE, verbose = TRUE)
  TV1 <- estimateTV(e,TV1,estCtrl)

  G1 <- garch(garchtype$general)
  G1 <- estimateGARCH(e,G1,estCtrl)
    
  MTV1 <- mtvgarch(e,TV1,G1)
  
}

#### Do first estimation iteration   ####
if(TRUE){
  
  # Create e, TV1 & G1 using the code above
  # make an original copy of the data:
  e_orig <- e
  estCtrl <- list(calcSE = TRUE, verbose = TRUE)
  
  MTV1 <- mtvgarch(e,TV1,G1)

  # Filter out Garch & estimate TV component
  if(TRUE) {
    
    lG1 <- MTV1@garch
    e <- e_orig/sqrt(lG1@h)
    
    lT1 <- MTV1@tv
    summary(lT1)
    ll.target <- getTargetValue(e,lT1)
    
    lT1$delta0 <- lT1$Estimated$delta0
    lT1$pars["deltaN",1] <- 1.0
    lT1$pars["speedN",1] <- 3.0
    lT1$pars["locN1",1] <- 0.1
    #lT1$pars["locN2",1] <- 0
    
    optCtrl <- lT1$optimcontrol
    optCtrl$reltol <- 1e-9
    optCtrl$ndeps <- c(1e-7,1e-7,1e-7)
    optCtrl$parscale <- c(3,1,1)
    lT1$optimcontrol <- optCtrl
    
    lT1 <- estimateTV(e,lT1,estCtrl)
    summary(lT1)  
    plot(lT1)
    
    # Check that the LL is higher than the target:
    lT1$Estimated$value > ll.target 
    
  }
  
  # Filter out TV & estimate GARCH component
  if(TRUE) {
    e <- e_orig/sqrt(lT1@g)
    
    lG1 <- MTV1@garch
    summary(lG1)
    ll.target <- getTargetValue(e,garchObj = lG1)
    
    lG1$pars["omega",1] <- 0.10
    lG1$pars["alpha",1] <- 0.05
    lG1$pars["beta",1] <- 0.80
    #lG1$pars["gamma",1] <- 0.02
    
    optCtrl <- lG1$optimcontrol
    optCtrl$reltol <- 1e-9
    optCtrl$ndeps <- c(1e-7,1e-7,1e-7)
    optCtrl$parscale <- c(1,1,9)
    lG1$optimcontrol <- optCtrl
    
    lG1 <- estimateGARCH(e,lG1,estCtrl)
    summary(lG1)  
    plot(lG1)
    
    # Check that the LL is higher than the target:
    lG1$Estimated$value > ll.target 
    
  }
  
  # Now call the calculate liklihood method for mtvgarch
  
  MTV1 <- calcLoglik(MTV1,lT1,lG1)
  
}



